// Drift camera mod by Miner34, Licensed under the MIT license.

const float pi = 3.141592f;

const float camera_distance = 5.0f; // Horizontal distance from vehicle origin
const float camera_height = 2.5f; // Vertical distance from vehicle origin
const float min_ground_distance = 1.0f; // Minimum distance from terrain

float camera_rot;
float rotation_diff;
float vehicle_speed;
vector3 vehicle_position;
vector3 camera_target;
vector3 camera_position;
vector3 vehicle_position_old;
vector3 vehicle_velocity;


void frameStep(float dt) // Executed every frame
{
    if (game.getCurrentTruck() != null) // Only run this part of the code if inside a vehicle
    {
        vehicle_position_old = vehicle_position;
        vehicle_position = game.getCurrentTruck().getHandle().getPosition(); // Get current vehicle origin
        vehicle_velocity = vehicle_position - vehicle_position_old;
        vehicle_speed = sqrt(pow(vehicle_position[0] - vehicle_position_old[0], 2) + pow(vehicle_position[2] - vehicle_position_old[2], 2)); // Calculate vehicle horizontal speed
        if (vehicle_speed > 0.02f or vehicle_speed < -0.02f) // Only executed if vehicle speed is higher than 0.02 meters per frame, reduces jittering
        {
            rotation_diff = atan2(-vehicle_velocity[0], -vehicle_velocity[2]) - camera_rot;
            while (rotation_diff > pi)
            {
                rotation_diff -= pi*2;
            }
            while (rotation_diff < -pi)
            {
                rotation_diff += pi*2;
            }
            camera_rot = camera_rot + rotation_diff * 5 * dt;
        }
        camera_position = vector3((camera_distance * sin(camera_rot)) + vehicle_position[0], vehicle_position[1] + camera_height, (camera_distance * cos(camera_rot)) + vehicle_position[2]);
        if (camera_position[1] < game.getGroundHeight(camera_position)) // If the camera is trying to go underground
        {
            camera_position = vector3(camera_position[0], game.getGroundHeight(camera_position) + min_ground_distance, camera_position[2]); // Set Y position to ground level + min_ground_distance
        }
        game.setCameraPosition(camera_position);
        game.cameraLookAt(vehicle_position);
    }
}
